@@ -2,6 +2,7 @@ import os
 import re
 from base64 import b64decode
 import requests
+from django.core.handlers.wsgi import WSGIRequest
 
 from cryptography.hazmat.primitives.serialization import load_pem_public_key
 from cryptography.hazmat.backends import default_backend
@@ -9,6 +10,7 @@ from cryptography.hazmat.backends import default_backend
 from passageidentity.errors import PassageError
 
 TOKEN_TYPE = "Bearer"
+BASE_URL = "https://api.passage.id/v1/apps/"
 
 """
 Helper function to extract the JWT from an Authorization header. 
@@ -35,16 +37,23 @@ def getAuthTokenFromRequest(request, auth_strategy):
         except (AttributeError, IndexError):
             return None
     else:
-        if 'psg_auth_token' not in request.cookies.keys():
-            raise PassageError("No Passage authentication token.")
-        return request.cookies['psg_auth_token']
+        # check if the request is from Django
+        is_django = isinstance(request,WSGIRequest)
+        if is_django:
+            if 'psg_auth_token' not in request.COOKIES.keys():
+                raise PassageError("No Passage authentication token.")
+            return request.COOKIES['psg_auth_token']
+        else: # assume its Flask in this case
+            if 'psg_auth_token' not in request.cookies.keys():
+                raise PassageError("No Passage authentication token.")
+            return request.cookies['psg_auth_token']
 
 """
 Helper function to fetch the public key for the given app id from Passage
 """
 def fetchPublicKey(app_id):
     # unauthenticated request to get the public key
-    r = requests.get("https://api.passage.id/v1/apps/" + app_id)
+    r = requests.get(BASE_URL + app_id)
 
     # check response code
     if r.status_code != 200:
