@@ -1,21 +1,50 @@
 import jwt, json, requests
 from datetime import datetime
-from typing import Union, TypedDict
+
+import sys
+
+if sys.version_info >= (3, 8):
+    from typing import TypedDict, Union
+else:
+    from typing_extensions import TypeDict, Union
 
 from requests.sessions import Request
 from passageidentity.helper import extractToken, getAuthTokenFromRequest, fetchPublicKey
 from passageidentity.errors import PassageError
+from enum import Enum
 
 PUBKEY_CACHE = {}
+BASE_URL = "https://api.passage.id/v1/apps/"
+
+class UserStatus(Enum):
+    ACTIVE = "active"
+    INACTIVE = "inactive"
+    PENDING = "pending"
+
+class ChannelType(Enum):
+    EMAIL = "email"
+    PHONE = "phone"
 
 class Passage():
     COOKIE_AUTH = 1
     HEADER_AUTH = 2
 
+    class PassageMagicLinkType(TypedDict, total=False):
+        id: str
+        secret: str
+        activated: bool
+        user_id: str
+        app_id: str
+        identifier: str
+        type: str
+        redirect_url: str
+        url: str
+        ttl: int
+
     class PassageUserType(TypedDict):
         created_at: str
         updated_at: str
-        active: bool
+        status: UserStatus
         email_verified: bool
         email: str
         phone: str
@@ -46,7 +75,7 @@ class Passage():
     
 
     """
-    Authenticate a Flask request that uses Passage for authentication.
+    Authenticate a Flask or Django request that uses Passage for authentication.
     This function will verify the JWT and return the user ID for the authenticated user, or throw
     a PassageError
     """ 
@@ -63,6 +92,53 @@ class Passage():
         except Exception as e:
             raise PassageError("JWT is not valid: " + str(e))
 
+    """
+    Authenticate a JWT from Passage. This function will verify the JWT and return the user ID 
+    for the authenticated user, or throw a PassageError. 
+    This function can be used to authenticate JWTs from Passage if they are not sent in a typical cookie or 
+    authorization header.
+    """ 
+    def authenticateJWT(self, token:str) -> Union[str, PassageError]:
+        # load and parse the JWT
+        try:
+            claims = jwt.decode(token, self.passage_pubkey, algorithms=["RS256"])
+            return claims["sub"]
+        except Exception as e:
+            raise PassageError("JWT is not valid: " + str(e)) 
+   
+    
+    class MagicLinkAttributes(TypedDict, total=False):
+        user_id: str
+        email: str
+        phone: str
+        channel: ChannelType
+        send: bool
+        magic_link_path: str
+        redirect_url: str
+        ttl: int
+
+    """
+    Create Passage MagicLink
+    """
+    def createMagicLink(self, magicLinkAttributes: MagicLinkAttributes) -> Union[PassageMagicLinkType, PassageError]:
+        # if no api key, fail
+        if self.passage_apikey == "":
+            raise PassageError("No Passage API key provided.")
+
+        header = {"Authorization": "Bearer " + self.passage_apikey}
+        try:
+            url = BASE_URL + self.app_id + "/magic-links"  
+            r = requests.post(url, data=json.dumps(magicLinkAttributes), headers=header)
+            if r.status_code != 201:
+                # get error message
+                message = r.json()["status"]
+                raise PassageError("Failed request to create magic link: " + message)
+
+            parsedResponse = r.json()["magic_link"]
+            return PassageMagicLink(parsedResponse["id"], parsedResponse)
+        except Exception as e:
+            raise PassageError("Could not create magic link")
+
 
     """
     Use Passage API to get info for a user, look up by user ID
@@ -73,7 +149,7 @@ class Passage():
 
         header = {"Authorization": "Bearer " + self.passage_apikey}
         try:
-            url = "https://api.passage.id/v1/apps/" + self.app_id + "/users/" + user_id
+            url = BASE_URL + self.app_id + "/users/" + user_id
             r = requests.get(url, headers=header)
 
             if r.status_code != 200:
@@ -94,7 +170,7 @@ class Passage():
 
         header = {"Authorization": "Bearer " + self.passage_apikey}
         try:
-            url = "https://api.passage.id/v1/apps/" + self.app_id + "/users/" + user_id + "/activate"  
+            url = BASE_URL + self.app_id + "/users/" + user_id + "/activate"  
             r = requests.patch(url, headers=header)
 
             if r.status_code != 200:
@@ -115,16 +191,16 @@ class Passage():
 
         header = {"Authorization": "Bearer " + self.passage_apikey}
         try:
-            url = "https://api.passage.id/v1/apps/" + self.app_id + "/users/" + user_id + "/deactivate"  
+            url = BASE_URL + self.app_id + "/users/" + user_id + "/deactivate"  
             r = requests.patch(url, headers=header)
 
             if r.status_code != 200:
                 # get error message
-                message = r.json()["status"]
-                raise PassageError("Failed request to deactivate user: " + message)
+                message = r.json()["error"]
+                raise PassageError(f"{message}")
             return PassageUser(user_id, r.json()["user"])
         except Exception as e:
-            raise PassageError("Could not deactivate user")
+            raise PassageError("Could not deactivate user: {e}")
 
 
     """
@@ -140,16 +216,16 @@ class Passage():
         
         header = {"Authorization": "Bearer " + self.passage_apikey}
         try:
-            url = "https://api.passage.id/v1/apps/" + self.app_id + "/users/" + user_id  
+            url = BASE_URL + self.app_id + "/users/" + user_id  
             r = requests.patch(url, headers=header, data=json.dumps(attributes))
             if r.status_code != 200:
                 # get error message
                 attributeKeys = ", ".join(attribute for attribute in attributes.keys())
-                message = r.json()["status"]
-                raise PassageError(f"Failed request to update user attributes ({attributeKeys}): {message}")
+                message = r.json()["error"]
+                raise PassageError(f"{message}")
             return PassageUser(user_id, r.json()["user"])
-        except Exception:
-            raise PassageError(f"Could not update user attributes")
+        except Exception as e:
+            raise PassageError(f"Could not update user attributes: {e}")
 
 
     """
@@ -161,7 +237,7 @@ class Passage():
 
         header = {"Authorization": "Bearer " + self.passage_apikey}
         try:
-            url = "https://api.passage.id/v1/apps/" + self.app_id + "/users/" + user_id  
+            url = BASE_URL + self.app_id + "/users/" + user_id  
             r = requests.delete(url, headers=header)
 
             if r.status_code != 200:
@@ -191,7 +267,7 @@ class Passage():
 
         header = {"Authorization": "Bearer " + self.passage_apikey}
         try:
-            url = "https://api.passage.id/v1/apps/" + self.app_id + "/users"  
+            url = BASE_URL + self.app_id + "/users"  
             r = requests.post(url, data=json.dumps(userAttributes), headers=header)
             if r.status_code != 201:
                 # get error message
@@ -203,13 +279,24 @@ class Passage():
         except Exception as e:
             raise PassageError("Could not create user")
 
-
+class PassageMagicLink:
+    def __init__(self, magic_link_id, fields={}):
+        self.id = magic_link_id
+        self.secret = fields["secret"]
+        self.activated = fields["secret"]
+        self.user_id = fields["user_id"]
+        self.app_id = fields["app_id"]
+        self.identifier = fields["identifier"]
+        self.type = fields["type"]
+        self.redirect_url = fields["redirect_url"]
+        self.url = fields["url"]
+        self.ttl = fields["ttl"]
 class PassageUser:
     def __init__(self, user_id, fields={}):
         self.id = user_id
         self.email = fields["email"]
         self.phone = fields["phone"]
-        self.active = fields["active"]
+        self.status = fields["status"]
         self.email_verified = fields["email_verified"]
         try:
             self.created_at = datetime.strptime(time_to_milliseconds(fields["created_at"]),"%Y-%m-%dT%H:%M:%S.%fZ")
